"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RepackTargetPlugin = void 0;
var _nodePath = _interopRequireDefault(require("node:path"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/**
 * {@link RepackTargetPlugin} configuration options.
 */

/**
 * Plugin for tweaking the JavaScript runtime code to account for React Native environment.
 *
 * Globally available APIs differ with React Native and other target's like Web, so there are some
 * tweaks necessary to make the final bundle runnable inside React Native's JavaScript VM.
 *
 * @category Webpack Plugin
 */
class RepackTargetPlugin {
  /**
   * Constructs new `RepackTargetPlugin`.
   *
   * @param config Plugin configuration options.
   */
  constructor(config) {
    this.config = config;
  }
  replaceRuntimeModule(module, content) {
    // webpack
    if ('getGeneratedCode' in module) {
      module.getGeneratedCode = () => content;
      return;
    }

    // rspack
    if (module.source !== undefined) {
      module.source.source = Buffer.from(content, 'utf-8');
    } else {
      // should never happen
      throw new Error('Module source is not available');
    }
  }
  /**
   * Apply the plugin.
   *
   * @param compiler Webpack compiler instance.
   */
  apply(compiler) {
    const Template = compiler.webpack.Template;
    const globalObject = 'self';
    compiler.options.target = false;
    compiler.options.output.chunkLoading = 'jsonp';
    compiler.options.output.chunkFormat = 'array-push';
    compiler.options.output.globalObject = globalObject;

    // Normalize global object.
    new compiler.webpack.BannerPlugin({
      raw: true,
      entryOnly: true,
      banner: Template.asString([`/******/ var ${globalObject} = ${globalObject} || this || new Function("return this")() || ({}); // repackGlobal'`, '/******/'])
    }).apply(compiler);

    // Replace React Native's HMRClient.js with custom Webpack-powered DevServerClient.
    new compiler.webpack.NormalModuleReplacementPlugin(/react-native.*?([/\\]+)Libraries([/\\]+)Utilities([/\\]+)HMRClient\.js$/, resource => {
      const request = require.resolve('../../modules/DevServerClient');
      const context = _nodePath.default.dirname(request);
      resource.request = request;
      resource.context = context;
      // @ts-ignore
      resource.createData.resource = request;
      // @ts-ignore
      resource.createData.context = context;
    }).apply(compiler);

    // ReactNativeTypes.js is flow type only module
    new compiler.webpack.NormalModuleReplacementPlugin(/react-native.*?([/\\]+)Libraries[/\\]Renderer[/\\]shims[/\\]ReactNativeTypes\.js$/, require.resolve('../../modules/EmptyModule')).apply(compiler);
    compiler.hooks.thisCompilation.tap('RepackTargetPlugin', compilation => {
      compilation.hooks.runtimeModule.tap('RepackTargetPlugin', (module, chunk) => {
        /**
         * We inject RePack's runtime modules only when load_script module is present.
         * This module is injected when:
         * 1. HMR is enabled
         * 2. Dynamic import is used anywhere in the project
         */
        if (module.name === 'load_script' || module.name === 'load script') {
          const loadScriptGlobal = compiler.webpack.RuntimeGlobals.loadScript;
          const loadScriptRuntimeModule = Template.asString([Template.getFunctionContent(require('./implementation/loadScript')).replaceAll('$loadScript$', loadScriptGlobal).replaceAll('$caller$', `'${chunk.id?.toString()}'`)]);
          const initRuntimeModule = Template.asString(['// Repack runtime initialization logic', Template.getFunctionContent(require('./implementation/init')).replaceAll('$globalObject$', globalObject).replaceAll('$hmrEnabled$', `${this.config?.hmr ?? false}`)]);

          // combine both runtime modules
          const repackRuntimeModule = `${loadScriptRuntimeModule}\n${initRuntimeModule}`;

          // inject runtime module
          this.replaceRuntimeModule(module, repackRuntimeModule.toString());
        }

        // Remove CSS runtime modules
        if (module.name === 'css_loading' || module.name === 'get css chunk filename') {
          this.replaceRuntimeModule(module, '// noop');
        }
      });
    });
  }
}
exports.RepackTargetPlugin = RepackTargetPlugin;
//# sourceMappingURL=RepackTargetPlugin.js.map